;; ======================================================================================================
;; Ausgeliehen von SReinhold. http://osd.cad.de/lisp_3d_08.htm
;;
;; ======================================================================================================
;; os 25.08.2003
;; M1 - Logical Table fuer Werkstoff und Dichte
;; M2 - Zuordnen von Dichte und Farbe - auch fuer Normteile
;; M3 - Sucht in Baugruppen oder unter Root nach Teilen mit einer bestimmten Dichte
;; M4 - Dichte ueber eine vorgegebene Masse berechnen und dem Teil zuordnen
;;
;; Ursprung aus http://osd.cad.de/ 'GD_properties.lsp'
;; Anpassungen:
;; - set_part_base_color war set_part_inst_color: Farbe wird neu der Grundfarbe zugeordnet, anstelle der Teilefarb
;; - set_part_base_density war set_part_inst_density : Dichte ...
;;
;; ======================================================================================================

(in-package :GD-package)
(use-package :OLI)

;; ======================================================================================================
;; M1 - Logical Table fuer Werkstoff und Dichte

(sd-create-logical-table 
 "Dichte-Farbe-Liste"
  :columns     '(:Benennung		:Dichte_1	:Abk	:Dichte  :Farbe2)
  
  :columnNames (sd-multi-lang-string
               '("Material"		"kg/dm^3"	"-"	"g/mm^3" "Farbe2")
                :german
               '("Werkstoff"		"kg/dm^3"	"-"	"g/mm^3" "Farbe2")
               );;sd-multi-lang-string

  :types       '(:string		:string		:string	:mass :string)
  :units       '(nil			nil		nil	:g nil)

  :contents   (sd-multi-lang-string
              '(("Aluminum"		"2,75"		"Al"		0.00275	"0.6,1,1")
               )
              :german
              '(
				("Aluminium"				"2,75"		"Al"				0.00275	"0.732,0.715,0.969")
				("Bronze"					"8,5"		"CuZn"				0.0085	"1,0.84,0.56")
				("Edelstahl Profile"		"7,85"		"1.4301-Profil"		0.00785	"1,0.28,0.0")
				("Edelstahlblech"			"7,85"		"1.4301-Bl"			0.00785	"0.32,0.32,1")
				("Blech verzinkt"			"7,85"		"verzinkt"			0.00785	"0.04,0.8,0.08")
				("Gusseisen"				"7,25"		"Gusseisen"			0.00725	"0.48,0.56,0.48")
                ("Kupfer"					"8,9" 		"Cu"				0.0089	"0.979,0.568,0.153")
				("Messing"					"8,50"		"CuZk"				0.0085	"1,0.7,0")
				("Nickel"					"8,91"		"Nickel"			0.00891	"0.52,0.8,0.48")
				("Stahl verzinkt"			"7,85"		"St-verz"			0.00785	"0.04,0.8,0.08")
				("Stahl schwarz"			"7,85"		"St-schwarz"		0.00785	"0.52,0.28,0.08")
				("Stahl Profil"				"7,85"		"St-Profil"			0.00785	"0.52,0.28,0.08")
                ("Zink"						"7,14"		"Zn"				0.00714	"0.6,0.8,1")
               )
              );;sd-multi-lang-string
);;sd-create-logical-table

;; =====================
(sd-create-display-table
     "Werkstoff-Dichte-Liste"
     :tableTitle         (sd-multi-lang-string "Material and Density"
                                       :german "Werkstoff und Dichte")
     :logicalTable       "Dichte-Farbe-Liste"
     :columns            '(:Benennung :Dichte_1 :Abk :Dichte :Farbe2)
     :applyColumns       '(:Abk :Dichte :Farbe2)
     :displayHeight      500
     :filterStatusLine   nil
     :selectionMode      :single-row 
     :applyAction        :default-tokens
);;sd-create-display-table

;; =====================
(sd-create-display-table
     "Dichte-Liste"
     :tableTitle         (sd-multi-lang-string "Material and Density"
                                       :german "Werkstoff und Dichte")
     :logicalTable       "Dichte-Farbe-Liste"
     :columns            '(:Benennung :Dichte_1 :Abk :Dichte :Farbe2)
     :applyColumns       '(:Dichte)
     :displayHeight      500
     :filterStatusLine   nil
     :selectionMode      :single-row 
     :applyAction        :default-tokens
);;sd-create-display-table


;; ======================================================================================================
;; M2 - Zuordnen von Dichte und Farbe - auch fuer Normteile

(sd-defdialog 'dichte_waehlen
  :dialog-title (sd-multi-lang-string "Density" 
                              :german "Dichte+Farbe")
							  :toolbox-button nil
  :variables '(
    (UI_PART 
        :value-type :part  
		:modifies nil
        :multiple-items t
        :title (sd-multi-lang-string "Part"
                             :german "Teil")
    )
    (ABK
        :value-type :string
        :title (sd-multi-lang-string "Werkstoff"
                             :german "Werkstoff")
        :show-input-tool (sd-show-display-table "Werkstoff-Dichte-Liste"
                          :position '("TOP-MENU-TOOLBOX-TB" :lefttop 70 50))
        :hide-input-tool (sd-hide-display-table  "Werkstoff-Dichte-Liste" 
                          :ignorePin t)
        :prompt-text (sd-multi-lang-string "Specify Material"
                                   :german "Werkstoff auswaehlen")
    );;ABK
    (DICHTE
        :value-type :positive-number
        :title (sd-multi-lang-string "Density"
                             :german "Dichte")
        :prompt-text (sd-multi-lang-string "Specify Denstiy"
                                   :german "Dichte eingeben")
    )

    (Farbe2
     :value-type :string
     :toggle-type :invisible
     :after-input (progn
		    (sd-set-variable-status "FARBE" :value (sd-rgb-to-color (read-from-string Farbe2)))
		    (setq FARBE (sd-rgb-to-color (read-from-string Farbe2)))
		    )
     )
    (FARBE
        :value-type :rgb-color
        :title (sd-multi-lang-string "Color"
                             :german "Farbe")
	;:after-input (display FARBE)
    )
	(Transparenz 	:expand-shrink (scaled_Trans )
					:expand-shrink-toggle-type :boolean)

    (scaled_Trans	:value-type      :scale
					:title "Standard - Halbtrans - Volltrans"
					:minimum         0
					:maximum         2
					:initial-value   0
					:after-input     (progn (setq Trans (/ scaled_Trans 2)))
					:initial-enable  t)
    ("Exemplar Eigenschaften entfernen")
      (cher_exemplarfarbe_entfernen 	:value-type    :grouped-boolean
										:title         "Farbe"
										:initial-value t
      )
      (cher_exemplardichte_entfernen 	:value-type    :grouped-boolean
										:title         "Dichte"
										:initial-value t
      )
	(cher_exemplartrans_entfernen 		:value-type    :grouped-boolean
										:title         "Transparenz"
										:initial-value t
      )

    ("")
    (AKTION_1
	:value-type :boolean
	:toggle-type :wide-toggle
	:title (sd-multi-lang-string "Density"
			     :german "Dichte")
	:initial-value nil
    )
    (AKTION_2
	:value-type :boolean
	:toggle-type :wide-toggle
	:title (sd-multi-lang-string "Density & Color"
			     :german "Dichte & Farbe")
	:initial-value t
    )
    ("")
    (AKTIONSSCHALTER
        :title (sd-multi-lang-string "apply"
                             :german "zuordnen")
        :toggle-type :wide-toggle
	:push-action (sd-call-cmds (l-dichte-action))
    );;AKTIONSSCHALTER
  );;variables

  :mutual-exclusion '(AKTION_1 AKTION_2)

  :local-functions '(
     (l-dichte-action ()
       (progn
            (if cher_exemplarfarbe_entfernen
              (part_prop UI_PART :COLOR :OFF)
            )
            (if cher_exemplardichte_entfernen
              (part_prop UI_PART :density :OFF)
            )
			(if cher_exemplartrans_entfernen
              (part_prop UI_PART :TRANS :NONE)
            )
		(sd-call-cmds (set_part_base_transparency :parts UI_PART :transp TRANS) :failure nil)
        (sd-call-cmds (set_part_base_density :parts UI_PART :dens DICHTE))
        (when AKTION_2
               (sd-call-cmds (set_part_base_color :parts UI_PART :color FARBE))
	)
       )
     );;l-dichte-action

  );;local-functions

  :ok-action '()
  :help-action '(progn 
	(display :show  )
	(display  "Diese Tool wurde von Adrian Fichte entwickelt!")
	(display  "0 = Standard (keine Transparenz")
	(display  "1 = Halbtransparenz")
	(display  "2 = Volltransparenz")
	(display  "www.pero-ag.de .:. adrian@36zero.de")
)

);;sd-defdialog


;; ======================================================================================================
;; M3 - Sucht in Baugruppen oder unter Root nach Teilen mit einer bestimmten Dichte

(sd-defdialog 'dichte-suchen-dialog
 :dialog-title (sd-multi-lang-string "Density = 1?"
                             :german "Dichte = ?")
							 :toolbox-button nil
 :variables
  '(
    ;;local
    (back-state :initial-value nil)
    ;;dialog
    (bgr    :value-type :assembly
            :title (sd-multi-lang-string "Assembly" :german "Baugruppe")
            :prompt-text (sd-multi-lang-string "Specify Assembly"
                                       :german "Baugruppe angeben")
            :modifies nil
    );;bgr
    (dichte :value-type :positive-number
            :title (sd-multi-lang-string "Density" :german "Dichte")
            :show-input-tool (sd-show-display-table "Dichte-Liste"
                              :position '("TOP-MENU-TOOLBOX-TB" :lefttop 70 50))
            :hide-input-tool (sd-hide-display-table  "Dichte-Liste" 
                              :ignorePin t)
            :prompt-text (sd-multi-lang-string "Specify Density"
                                       :german "Dichte eingeben")
            :initial-value 1
    );;dichte
    (beschr :value-type :boolean
            :toggle-type :wide-toggle
            :title (sd-multi-lang-string "Label" :german "Beschriften")
            :initial-value t
    );;beschr
    (keep   :value-type :boolean
            :toggle-type :wide-toggle
            :title (sd-multi-lang-string "keep Label" :german "Beschriftung behalten")
            :initial-value nil
    );;keep
    (next   :title (sd-multi-lang-string "Find" :german "Suchen")
            :toggle-type :wide-toggle
            :push-action (sd-call-cmds (next-action))
    );;next
   )
 :local-functions
  '((next-action ()
     (when (and back-state (not keep))
      (progn
       (sd-return-to-model-checkpoint back-state)
       (setf back-state nil)
      );;progn
     );;when
     (if (and bgr dichte)
      (progn
       (setf back-state (sd-set-model-checkpoint))
       (suche-dichte-in-bgr bgr dichte beschr)
      );;progn
      (sd-display-error (sd-multi-lang-string "Not all necessary inputs are specified!"
                                      :german "Es wurden nicht alle erforderlichen Eingaben gemacht!")
      );;sd-display-error
     );;if
    )
   )
 :cleanup-action
   '(when (and back-state (not keep)) (sd-return-to-model-checkpoint back-state))
 :ok-action '()
 :help-action '(sd-display-url (format nil "~A/sd_help_10.htm" mei::*GD_help_dir*))
)

;; durchsucht alle Teile nach der Dichte (dichte) und gibt eine
;; entsprechende Liste mit allen Teilen mit der entsprechende
;; Dichte aus!

(defun suche-dichte-in-bgr (teil_bgr dichte beschr)
 (let (teiletyp kind kinder teilgenau txt)
  (setf teiletyp (sel_item-type teil_bgr))
  (if (equal teiletyp *sd-part-seltype*)
   (progn
    (when (not (sd-inq-empty-part-p teil_bgr))
     (progn
      (setf teildichte (sd-inq-part-density teil_bgr))
      (if (= dichte teildichte)
       (progn
        (if (sd-inq-obj-contents-read-only-p teil_bgr)
         (display (sd-multi-lang-string
                           (format nil "Density ~,,,,,,'EG: ~a (read only)" teildichte (sd-inq-obj-pathname teil_bgr))
                   :german (format nil "Dichte ~,,,,,,'EG: ~a (read only)" teildichte (sd-inq-obj-pathname teil_bgr))
                  );;sd-multi-lang-string
         );;display
         (display (sd-multi-lang-string
                           (format nil "Density ~,,,,,,'EG: ~a" teildichte (sd-inq-obj-pathname teil_bgr))
                   :german (format nil "Dichte ~,,,,,,'EG: ~a" teildichte (sd-inq-obj-pathname teil_bgr))
                  );;sd-multi-lang-string
         );;display
        );;if
        (if beschr
         (progn
          (setf txt (format nil "Dichte ~,,,,,,'EG" teildichte))
          (when (not (sd-inq-obj-contents-read-only-p teil_bgr))
           (cocreate_3d_note :action :create :note txt :items teil_bgr)
          );;when
         );;progn
         nil
        );;if
       );;progn
       nil
      );;if
     );;progn
    );;when
   );;progn
   ;; else
   (when (equal teiletyp *sd-assembly-seltype*)
    (progn
     (setf kinder (sd-inq-obj-children teil_bgr))
     (dolist (kind kinder)
      (suche-dichte-in-bgr kind dichte beschr)
     ) ;; dolist
    ) ;; progn
   ) ;; when
  ) ;; if
 ) ;; let
)


;; ======================================================================================================
;; M4 - Dichte ueber eine vorgegebene Masse berechnen und dem Teil zuordnen

(sd-defdialog 'os_masse_dichte
  :dialog-title (sd-multi-lang-string "DensityMass"
                              :german "DichteMasse")
							  :toolbox-button nil
  :variables
  '(
    (TEIL
     :value-type :part
     :title (sd-multi-lang-string "Part"
                          :german "Teil")
     :prompt-text (sd-multi-lang-string "Specify Part"
                                :german "Teil angeben")
    )
    (MASSE
     :value-type :number
     :title (sd-multi-lang-string "Mass / g"
                          :german "Masse / g")
     :prompt-text (sd-multi-lang-string "Input Mass in g"
                                :german "Masse in g eingeben")
    )
   );variables
  :local-functions
   '(
     (dichte_ueber_masse_zuordnen ()
      (let (VOLUMEN DICHTE)
      ;; Volumen abfargen
      (setq VOLUMEN
        (sd-call-cmds
          (get_vol_prop
            :for_part
            :part_asmb (sd-inq-obj-pathname TEIL)
           :volume
          );get_vol_prop
          :failure nil
        );sd-call-cmds
      );;setq volume
      ;; Dichte berechnen
      (setq DICHTE (/ MASSE VOLUMEN))
      ;; Dichte zuordnen
      (sd-call-cmds (set_part_base_density :parts TEIL :dens DICHTE))
      ;;(sd-call-cmds (set_part_inst_density :parts TEIL :dens DICHTE))

      (display
        (sd-multi-lang-string
            (format nil "Dem Teil wurde eine Teiledichte von ~A g/mm3 zugeordnet" DICHTE)
           :german 
            (format nil "Dem Teil wurde eine Teiledichte von ~A g/mm3 zugeordnet" DICHTE)
        );;sd-multi-lang-string
      );;display
      );;let
    );;dichte_ueber_masse_zuordnen
   );;local-functions

  :ok-action '(dichte_ueber_masse_zuordnen)

);;sd-defdialog
